/*
 * $Id: interface.h,v 1.12 1999/05/29 22:13:13 masaki Exp $
 */

#ifndef _INTERFACE_H
#define _INTERFACE_H

/* Some of this code taken from Berkely routed software */

#include <mrt.h>
#include <trace.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>

#ifdef SOLARIS
#include <sys/sockio.h>
#endif

typedef struct _ll_addr_t {
    prefix_t *prefix;		/* interface addresess */
    prefix_t *broadcast;	/* destination or broadcast address
				   broadcast address is defined in case of v4
				   prefixlen will be ignored */
} ll_addr_t;

#define IFF_TUNNEL  0x80000000  /* tunnel interface (special treatment
                                   when adding a route into the kernel) */
#define IFF_QUERIER 0x40000000  /* igmp querier */

#define MAX_INTERFACES 256
#define MAX_INTERFACES 256
typedef struct _inteface_bitset_t {
        bitx_mask_t bits[(MAX_INTERFACES+BITX_NBITS-1)/BITX_NBITS];
} inteface_bitset_t;


typedef struct _interface_t {
#ifdef notdef
    u_long bit;			/* each interface assigned a unique bit */
#endif
    u_long flags;		/* interface flags (from the ioctl) */
    int mtu;			/* interface mtu (from the ioctl) */
    char name[IFNAMSIZ + 1];
    int index;			/* index number */
    int vif_index;		/* vif index number */
    u_long protocol_mask;	/* which protocols are on */

#ifdef notdef
/* the followings are RIP and RIPng deppendent */
    int dlist_in[PROTO_MAX];	/* list num for input filtering */
    int dlist_out[PROTO_MAX];	/* list num for output filtering */
    int metric_in;		/* input metric */
    int metric_out;		/* output metric */
    int default_pref;
#ifdef HAVE_IPV6
    int default_pref6;
#endif /* HAVE_IPV6 */
#endif /* notdef */

    LINKED_LIST *ll_addr;	/* address list */
    ll_addr_t *primary;		/* primary address for ipv4 */
#ifdef HAVE_IPV6
    ll_addr_t *primary6;	/* primary address for ipv6 (global) */
    ll_addr_t *link_local;	/* ipv6 link local address */
#endif /* HAVE_IPV6 */

    HASH_TABLE *membership;		/* connected groups */
#ifdef HAVE_IPV6
    HASH_TABLE *membership6;		/* connected groups */
#endif /* HAVE_IPV6 */
    mtimer_t *igmp_query_timer;
    int igmp_query_count;
    prefix_t *igmp_querier_prefix;
    mtimer_t *igmp_querier_timer;
    LINKED_LIST *ll_query_reqs;
    int igmp_refs;		/* how many protocols requested */
#ifdef HAVE_IPV6
    int igmp_refs6;		/* how many protocols requested */
#endif /* HAVE_IPV6 */

    prefix_t *tunnel_source;
    prefix_t *tunnel_destination;
} interface_t;


typedef struct _interface_master_t {
    int number;		/* number of interfaces */
    interface_t *default_interface;
#ifdef HAVE_IPV6
    interface_t *default_interface6;
#endif				/* HAVE_IPV6 */
    trace_t *trace;
    LINKED_LIST *ll_interfaces;	/* IPv4 & v6 interfaces */
    LINKED_LIST *ll_tunnels;	/* IPv4 & v6 tunnels */
    int max_mtu;		/* maximum MTU */
    interface_t *index2if[MAX_INTERFACES];
    int sockfd;			/* general purpose socket (v4) */
    int sockfd6;		/* general purpose socket (v6) */
    pthread_mutex_t mutex_lock;
} interface_master_t;

extern interface_master_t *INTERFACE_MASTER;

/* This may conflict with other MSG_XXX flags defined in the system */
#define MSG_DONTCARE  0x80000000  /* interface info is not required */
#define MSG_MAYIGMP   0x40000000  /* a packet may be igmp kernel msg */

/* public functions */
interface_t *new_interface (char *name, u_long flags, int mtu, int index);
interface_t *add_addr_to_interface (interface_t * interface, int family,
				    void *addr, int len, void *broadcast);
interface_t *update_addr_of_interface (int cmd,
                          interface_t * interface, int family,
                          void *addr, int bitlen, void *broadcast);
int read_interfaces ();

interface_t *find_interface (prefix_t * prefix);
interface_t *find_interface_flags (prefix_t * prefix, u_long flags);
interface_t *find_interface_byname (char *name);
LINKED_LIST *find_interface_byname_all (char *name);
interface_t *find_interface_byindex (int index);
interface_t *find_interface_local (prefix_t * prefix);
LINKED_LIST *find_network (prefix_t * prefix);
interface_t *local_interface (int familly, void *cp);
int is_prefix_on (prefix_t *prefix, interface_t *interface);
int init_interfaces (trace_t * ltrace);
interface_t *find_tunnel_interface (prefix_t * prefix);

int show_interfaces (uii_connection_t *uii);
int mask2len (void *mask, int bytes);
u_char *len2mask (int bitlen, void *mask, int bytes);

void kernel_update_route (prefix_t *dest, 
			  generic_attr_t *new_attr, generic_attr_t *old_attr, int pref);
int sys_kernel_update_route (prefix_t *dest,
                         prefix_t *next_hop, prefix_t *old_hop, 
			 int index, int oldindex);
int kernel_init (void);
void kernel_read_rt_table (int seconds);
int sys_kernel_read_rt_table (void);
int add_kernel_route (int family, void *dest, void *nhop, int masklen, 
		      int index);
int update_kernel_route (int cmd, int family, void *dest, void *nhop, 
			 int masklen, int index, int proto);
char *print_iflags (char *tmpx, int len, u_long flags);
void add_interfaces_to_rib (void);

/* socket.c */

int socket_open (int family, int type, int proto);
int socket_reuse (int sockfd, int yes);
int socket_broadcast (int sockfd, int yes);
int socket_rcvbuf (int sockfd, int size);
int ip_hdrincl (int sockfd, int yes);
int ip_multicast_loop (int sockfd, int yes);
int ip_multicast_hops (int sockfd, int hops);
int ip_recvdstaddr (int sockfd, int yes);
int ip_recvttl (int sockfd, int yes);
int ip_recvinterface (int sockfd, int yes);

#ifdef HAVE_IPV6
int ipv6_multicast_loop (int sockfd, int yes);
int ipv6_pktinfo (int sockfd, int yes);
int ipv6_recvdstaddr (int sockfd, int yes);
int ipv6_recvhops (int sockfd, int yes);
int ipv6_unicast_hops (int sockfd, int hops);
int ipv6_multicast_loop (int sockfd, int yes);
int ipv6_multicast_hops (int sockfd, int hops);
int ipv6_multicast_if (int sockfd, interface_t *interface);
#endif /* HAVE_IPV6 */
int join_leave_group (int sockfd, interface_t *interface, prefix_t *prefix, 
		      int join);
int ip_multicast_if (int sockfd, interface_t *interface);
int send_multicast (int sockfd, char *msg, int len, int flags, 
	            prefix_t *prefix, interface_t *interface);
int recvmsgfrom (int sockfd, char *buffer, int buflen, int flags,
	         prefix_t **prefix_p, int *port_p, interface_t **interface_p);
int send_packet (int sockfd, char *msg, int len, int flags,
             prefix_t *prefix, int port, interface_t *interface, int flowinfo);
#ifdef HAVE_IFF_MULTICAST
int mc_mrtinit (void);
int mc_mrtversion (void);
int mc_mrtdone (void);
int mc_add_vif (interface_t *interface);
extern interface_t *vif2if[];
extern int num_vif;
int mc_add_vif (interface_t *interface);
int mc_del_vif (interface_t *interface);
int mc_add_mfc (prefix_t *group, prefix_t *origin, interface_t *parent,
    u_char *ttls);
int mc_del_mfc (prefix_t *group, prefix_t *origin);
int mc_assert (int yes);
#ifdef HAVE_IPV6
int mc6_mrtinit (void);
int mc6_mrtdone (void);
#endif /* HAVE_IPV6 */
#endif /* HAVE_IFF_MULTICAST */

#endif /* INTERFACE_H */
